/*

fits_calc.cpp version 1.0

Copyright Yann OLLIVIER 1996

This software may be freely distributed as is, including this whole notice.

It may be modified, but any modification should include this whole 
notice as is and a description of the changes made.

This software may not be sold.

This software or any modified version of it may be freely used in free
programs. The program should include a copy of this whole notice.
If you want to use it in a program you sell, contact me

This software comes with absolutely no warranty.

*/

#include "fits.h"

// Compatibility with long double-typed functions
#define atanl atan
#define asinl asin
#define acosl acos
#define expl exp
#define logl log
#define powl pow
#define pow10l(x) pow(10,x)
#define fabsl fabs
#define cosl cos
#define sinl sin
#define tanl tan
#define fmodl fmod
#define sqrtl sqrt
#define atan2l atan2

TStat Stats(PTMatrix PM1,PTMatrix PM2,int start,int end)
{
  double m1=0,s1=0,m2=0,s2=0,cov=0,eqm=0;
  for(int i=start;i<=end;i++){
    m1+=(*PM1)(i);m2+=(*PM2)(i);
    s1+=(*PM1)(i)*(*PM1)(i);s2+=(*PM2)(i)*(*PM2)(i);
    cov+=(*PM1)(i)*(*PM2)(i);
    eqm+=((*PM1)(i)-(*PM2)(i))*((*PM1)(i)-(*PM2)(i));
  };
  m1/=(end-start+1);m2/=(end-start+1);eqm/=(end-start+1);eqm=sqrtl(eqm);
  TStat Stat;Stat.moy1=m1;Stat.moy2=m2;Stat.eqm=eqm;
  Stat.s1=sqrtl(s1/(end-start+1)-m1*m1);
  Stat.s2=sqrtl(s2/(end-start+1)-m2*m2);
  Stat.cov=cov/(end-start+1)-m1*m2;
  if(Stat.s1*Stat.s2)Stat.corr=Stat.cov/Stat.s1/Stat.s2;else Stat.corr=0;
  return Stat;
}

double Coeff(PTMatrix PM1,PTMatrix PM2)
{
  double x=0,y=0;int n=PM1->nl,i=0;
  for(;i<n;i++){x+=(*PM1)(i)*(*PM1)(i);y+=(*PM1)(i)*(*PM2)(i);};
  if(x==0)return 0;else return y/x;
}

PTMatrix Derive(PTMatrix PM1,PTMatrix PM2)
{
  TMatrix Y(3,1),X(3,3),A(3,1);
  PTMatrix PDer=new TMatrix(PM1->nl-2,1);
  int i,j;
  for(i=1;i<PM1->nl-1;i++){
    for(j=-1;j<=1;j++){
      Y(j+1,0)=(*PM2)(i+j);
      X(j+1,0)=1;X(j+1,1)=(*PM1)(i+j);
      X(j+1,2)=X(j+1,1)*(*PM1)(i+j);
    }
    A=~X*Y;
    (*PDer)(i-1)=A(1)+2*A(2)*X(1,1);
  }
  return PDer;
}

PTMatrix Integre(PTMatrix PM1,PTMatrix PM2)
{
  PTMatrix PInt=new TMatrix(PM1->nl-2,1);
  (*PInt)(0,0)=0;
  TMatrix Y(4,1),X(4,4),A(4,1);
  int i,j;
  for(i=1;i<PM1->nl-2;i++){
    for(j=-1;j<=2;j++){
      Y(j+1)=(*PM2)(i+j);
      X(j+1,0)=1;X(j+1,1)=(*PM1)(i+j);
      X(j+1,2)=X(j+1,1)*(*PM1)(i+j);
      X(j+1,3)=X(j+1,2)*(*PM1)(i+j);
    }
    A=~X*Y;
    (*PInt)(i)=(*PInt)(i-1)+A(3)/4*(X(2,3)*X(2,1)-X(1,3)*X(1,1))+
      A(2)/3*(X(2,3)-X(1,3))+A(1)/2*(X(2,2)-X(1,2))+A(0)*(X(2,1)-X(1,1));
  }
  return PInt;
}

void TModLin::Etablit(PTMatrix PX,PTMatrix PY)
{
  imp=0;if(!ai&&!bi){
    stat=Stats(PX,PY,0,PX->nl-1);
    if(stat.s1==0){imp=1;expr="Linear model unadapted.";return;};
    a=stat.cov/stat.s1/stat.s1;
    b=stat.moy2-stat.moy1*a;
  }else if(ai&&!bi){
    if(PX->nl==0){imp=1;expr="Linear model unadapted.";return;};
    double x=0,y=0;
    for(int i=0;i<PX->nl;i++){x+=a*(*PX)(i);y+=(*PY)(i);};
    b=(y-x)/PX->nl;
  }else if(!ai&&bi){
    double x=0,y=0;
    for(int i=0;i<PX->nl;i++){
      x+=(*PX)(i)*(*PX)(i);
      y+=(*PX)(i)*((*PY)(i)-b);
    };
    if(x==0){imp=1;expr="Linear model unadapted.";return;};
    a=y/x;
  };
  TMatrix B(PX->nl,1,b);
  TMatrix Y2=a*(*PX)+B;
  stat=Stats(PY,&Y2,0,PX->nl-1);
  delete[] expr;expr=new char[80];
  sprintf(expr,"Y=aX+b\na=%g\nb=%g",a,b);
}

TVal TModLin::f(double x)
{
  TVal Val;if(imp)Val.IsDefined=0;else{
    Val.IsDefined=1;
    Val.v=a*x+b;};
  return Val;
}

void TModPoly::Etablit(PTMatrix PX,PTMatrix PY)
{
  imp=0;
  int d,i,n=0,i2;for(d=0;d<=deg;d++)if(ai[d]==0)n++;
  if(n>PX->nl){
    imp=1;delete[]expr;expr=new char[50];
    sprintf(expr,"Polynomial model of degree %i unadapted.",deg);
    return;};
  TMatrix Z(PX->nl,n),Y2(PX->nl,1);
  double x;for(i=0;i<PX->nl;i++){
    i2=0;x=1;Y2(i)=(*PY)(i);
    for(d=0;d<=deg;d++,x*=(*PX)(i))
      if(ai[d]==0)Z(i,i2++)=x;else Y2(i)-=x*a[d];
  };
  TMatrix Y3(PX->nl,1);
  if(n!=0){TMatrix Z2=~(!Z*Z);
  if(Z2==MatrixNull){
    imp=1;delete[]expr;expr=new char[50];
    sprintf(expr,"Polynomial model of degree %i unadapted.",deg);
    return;};
  TMatrix B=Z2*!Z*Y2;i=0;
  for(d=0;d<n;d++){for(;ai[i];i++);a[i++]=B(d);};
  Y3=(*PY)-Y2+Z*B;
  }else Y3=(*PY)-Y2;
  stat=Stats(PY,&Y3,0,PX->nl-1); 
  delete[] expr;char s[40]="";i=0;sprintf(s,"n=%i\n",deg);i+=strlen(s);
  for(d=0;d<=deg;d++){sprintf(s,"a%i=%g\n",d,a[d]);i+=strlen(s);}
  expr=new char[30+i];
  sprintf(expr,"Y = an X^n +...+ a1 X + a0\nn=%i\n",deg);
  for(d=0;d<=deg;d++){sprintf(s,"a%i=%g\n",d,a[d]);strcat(expr,s);}
}

TVal TModPoly::f(double x)
{TVal Val;if(imp)Val.IsDefined=0;else{
	Val.IsDefined=1;double xn=1;Val.v=0;
	for(int d=0;d<=deg;d++,xn*=x)Val.v+=a[d]*xn;
	};
return Val;
}

void TModExp1::Etablit(PTMatrix PX,PTMatrix PY)
{int i;imp=0;
if(!bi){TMatrix B=MatrixNull;
	if(!ci){
		TMatrix Z(PX->nl-2,3);
		for(i=0;i<PX->nl-2;i++){
			Z(i,0)=1;Z(i,1)=(*PX)(i+1);Z(i,2)=(*PY)(i+1);};
		PTMatrix PIY=Integre(PX,PY);
		TMatrix Z2=~(!Z*Z);if(Z2==MatrixNull){imp=1;expr="Exponential model unadapted.";return;};
		B=Z2*!Z*(*PIY);c=B(1);};
	for(i=0;i<PX->nl;i++)if((*PY)(i)<=c)break;
	if(i==PX->nl){TMatrix Y2(PX->nl,1);
		for(i=0;i<PX->nl;i++){
			Y2(i)=logl((*PY)(i)-c);
      if(ai&&a>0)Y2(i)-=logl(a);};
		if(ai&&a>0)b=Coeff(PX,&Y2);else{stat=Stats(PX,&Y2,0,PX->nl-1);
			if(stat.s1==0){imp=1;expr="Exponential model unadapted.";return;};
			b=stat.cov/stat.s1/stat.s1;};
		}else
		{for(i=0;i<PX->nl;i++)if((*PY)(i)>=c)break;
		if(i==PX->nl){TMatrix Y2(PX->nl,1);
		for(i=0;i<PX->nl;i++){
			Y2(i)=logl(-(*PY)(i)+c);
      if(ai&&a<0)Y2(i)-=logl(-a);};
		if(ai&&a<0)b=Coeff(PX,&Y2);else{stat=Stats(PX,&Y2,0,PX->nl-1);
			if(stat.s1==0){imp=1;expr="Exponential model unadapted.";return;};
			b=stat.cov/stat.s1/stat.s1;};
		}else
		{if(!ci){if(B(2)==0){imp=1;expr="Exponential model unadapted.";return;};
			b=1/B(2);}
		else{
			PTMatrix PIY=Integre(PX,PY);TMatrix Z(PX->nl-2,2);
			for(i=0;i<PX->nl-2;i++){
				(*PIY)(i)-=c*(*PX)(i+1);
				Z(i,0)=1;Z(i,1)=(*PY)(i+1);};
			TMatrix Z2=~(!Z*Z);if(Z2==MatrixNull){imp=1;expr="Exponential model unadapted.";return;};
			B=Z2*!Z*(*PIY);
			if(B(1)==0){imp=1;expr="Exponential model unadapted.";return;};
			b=1/B(1);};
		};};
	};
if(ci&&!ai){
	double x=0,y=0;
	for(i=0;i<PX->nl;i++){x+=expl(2*b*(*PX)(i));y+=((*PY)(i)-c)*expl(b*(*PX)(i));};
	if(x==0){imp=1;expr="Exponential model unadapted.";return;};
	a=y/x;
	}else if(!ci&&ai){
	double x=0,y=0;
	for(i=0;i<PX->nl;i++){x+=a*expl(b*(*PX)(i));y+=(*PY)(i);};
	if(PX->nl==0){imp=1;expr="Exponential model unadapted.";return;};
	c=(y-x)/PX->nl;
	}else if(!ci&&!ai){
	TMatrix X2(PX->nl,1);
  for(i=0;i<PX->nl;i++)X2(i)=expl(b*(*PX)(i));
	stat=Stats(&X2,PY,0,PX->nl-1);
	if(stat.s1==0){imp=1;expr="Exponential model unadapted.";return;};
	a=stat.cov/stat.s1/stat.s1;c=stat.moy2-a*stat.moy1;
  };
TMatrix Y3(PX->nl,1);
for(i=0;i<PX->nl;i++)Y3(i)=a*expl(b*(*PX)(i))+c;
stat=Stats(PY,&Y3,0,PX->nl-1);
delete[] expr;expr=new char[130];
sprintf(expr,"Y = a exp(b X) + c\na=%g\nb=%g\nc=%g",a,b,c);
}

TVal TModExp1::f(double x)
{TVal Val;if(imp)Val.IsDefined=0;else{
	Val.IsDefined=1;
	Val.v=a*expl(x*b)+c;};
return Val;
}

void TModExp3::Etablit(PTMatrix PX,PTMatrix PY)
{imp=0;
int d,i,n=0,i2;for(d=0;d<=deg;d++)if(ai[d]==0)n++;
if(n+1>PX->nl){imp=1;expr="Exponential model unadapted.";return;};
if(!bi){PTMatrix PIY=Integre(PX,PY);
	TMatrix Y2(PX->nl-2,1),Z(PX->nl-2,3*(deg+1)-1);
	for(i=0;i<PX->nl-2;i++){
		double x=1;
		for(d=0;d<=deg;d++,x*=(*PX)(i+1)){
			Z(i,d)=x;Z(i,d+deg+1)=(*PY)(i+1)*x;
			if(d<deg)Z(i,d+2*deg+2)=(*PIY)(i)*x;else Y2(i)=(*PIY)(i)*x;
			};
		};
	TMatrix Z2=~(!Z*Z);
	if(Z2==MatrixNull){imp=1;expr="Exponential model unadapted.";return;};
	TMatrix B=Z2*!Z*Y2;
	if(B(2*deg+1)==0){imp=1;expr="Exponential model unadapted.";return;};
	b=1/B(2*deg+1);};
TMatrix Z(PX->nl,n),Y2(PX->nl,1);
double x;for(i=0;i<PX->nl;i++){
	i2=0;x=expl(b*(*PX)(i));Y2(i)=(*PY)(i);
	for(d=0;d<=deg;d++,x*=(*PX)(i))
		if(ai[d]==0)Z(i,i2++)=x;else Y2(i)-=x*a[d];
  };
TMatrix Y3(PX->nl,1);
if(n!=0){TMatrix Z2=~(!Z*Z);
	if(Z2==MatrixNull){imp=1;expr="Exponential model unadapted.";return;};
	TMatrix B=Z2*!Z*Y2;i=0;
	for(d=0;d<n;d++){for(;ai[i];i++);a[i++]=B(d);};
	Y3=(*PY)-Y2+Z*B;
	}else Y3=(*PY)-Y2;
stat=Stats(PY,&Y3,0,PX->nl-1); 
delete[] expr;char s[50]="";i=0;sprintf(s,"b=%g\nn=%i\n",b,deg);i+=strlen(s);
for(d=0;d<=deg;d++){sprintf(s,"a%i=%g\n",d,a[d]);i+=strlen(s);}
expr=new char[50+i];
sprintf(expr,"Y = exp(b X) (an X^n +...+ a1 X + a0)\nb=%g\nn=%i\n",b,deg);
for(d=0;d<=deg;d++){sprintf(s,"a%i=%g\n",d,a[d]);strcat(expr,s);}
}

TVal TModExp3::f(double x)
{TVal Val;if(imp)Val.IsDefined=0;else{
	Val.IsDefined=1;double xn=1;Val.v=0;
	for(int d=0;d<=deg;d++,xn*=x)Val.v+=a[d]*xn;
	Val.v*=expl(b*x);
	};
return Val;
}

void TModLog1::Etablit(PTMatrix PX,PTMatrix PY)
{int i;imp=0;
TMatrix Y3(PX->nl,1);
if(bi&&ci){for(i=0;i<PX->nl;i++)if(b*(*PX)(i)+c<=0)
		{imp=1;expr="Logarithmic model unadapted.";return;};
	for(i=0;i<PX->nl;i++)Y3(i)=logl(b*(*PX)(i)+c);
	if(!ai)a=Coeff(&Y3,PY);
	for(i=0;i<PX->nl;i++)Y3(i)*=a;
  }else{
if(!ai){PTMatrix PIX=Integre(PY,PX);
	if(!ci){TMatrix Z(PX->nl-2,3);
		for(i=0;i<PX->nl-2;i++){
			Z(i,0)=1;Z(i,1)=(*PY)(i+1);Z(i,2)=(*PX)(i+1);};
		TMatrix Z2=~(!Z*Z);if(Z2==MatrixNull){imp=1;expr="Logarithmic model unadapted.";return;};
		TMatrix B=Z2*!Z*(*PIX);
		if(B(2)==0){imp=1;expr="Logarithmic model unadapted.";return;};
		a=B(2);
		}else{TMatrix Z(PX->nl-2,2);
		for(i=0;i<PX->nl-2;i++){(*PIX)(i)+=c*(*PY)(i+1);
			Z(i,0)=1;Z(i,1)=(*PX)(i+1);};
		TMatrix Z2=~(!Z*Z);if(Z2==MatrixNull){imp=1;expr="Logarithmic model unadapted.";return;};
		TMatrix B=Z2*!Z*(*PIX);
		if(B(1)==0){imp=1;expr="Logarithmic model unadapted.";return;};
		a=B(1);};
	}else if(a==0){imp=1;expr="Logarithmic model unadapted.";return;};
if(!bi&&!ci){for(i=0;i<PX->nl;i++)Y3(i)=expl((*PY)(i)/a);
	stat=Stats(&Y3,PX,0,PX->nl-1);
	if(stat.cov==0||stat.s1==0){imp=1;expr="Logarithmic model unadapted.";return;};
	b=stat.s1*stat.s1/stat.cov;c=-stat.moy2*b+stat.moy1;
	for(i=0;i<PX->nl;i++)if(b*(*PX)(i)+c<=0){c=-b*(*PX)(i)+0.001;};
	}else if(bi&&!ci){if(b==0){imp=1;expr="Logarithmic model unadapted.";return;};
	for(i=0;i<PX->nl;i++)Y3(i)=expl((*PY)(i)/a)/b;
	stat=Stats(&Y3,PX,0,PX->nl-1);
	c=b*(stat.moy1-stat.moy2);
	for(i=0;i<PX->nl;i++)if(b*(*PX)(i)+c<=0){c=-b*(*PX)(i)+0.001;};
	}else if(ci&&!bi){
	for(i=0;i<PX->nl;i++)Y3(i)=expl((*PY)(i)/a)-c;
	b=Coeff(PX,&Y3);
	for(i=0;i<PX->nl;i++)if(b*(*PX)(i)+c<=0)
		{imp=1;expr="Logarithmic model unadapted.";return;};
	};
	for(i=0;i<PX->nl;i++)Y3(i)=a*logl(b*(*PX)(i)+c);
};
stat=Stats(PY,&Y3,0,PX->nl-1);
delete[] expr;expr=new char[130];
sprintf(expr,"Y = a log(b X + c)\na=%g\nb=%g\nc=%g",a,b,c);
}

TVal TModLog1::f(double x)
{TVal Val;if(imp||b*x+c<=0)Val.IsDefined=0;else{
	Val.v=a*logl(b*x+c);
	Val.IsDefined=1;
	};
return Val;
}

void TModPuis::Etablit(PTMatrix PX,PTMatrix PY)
{int i;imp=0;if(ci&&c==-1){imp=1;expr="Power model doesn't work with c=-1. Sorry.";return;};
if(ai&&bi&&ci&&di){}else{
	if(bi&&di){for(i=0;i<PX->nl;i++)if((*PX)(i)+b<0)
				{imp=1;expr="Power model unadapted.";return;};
			for(i=0;i<PX->nl;i++)if((*PY)(i)-d<=0||(*PX)(i)+b<=0)break;
			if(i==PX->nl){
				TMatrix X2(PX->nl,1),Y2(PX->nl,1);
				for(i=0;i<PX->nl;i++){X2(i)=logl((*PX)(i)+b);Y2(i)=logl((*PY)(i)-d);};
				stat=Stats(&X2,&Y2,0,PX->nl-1);
				if(stat.s1==0){imp=1;expr="Power model unadapted.";return;};
				c=stat.cov/stat.s1/stat.s1;
				}else{
				for(i=0;i<PX->nl;i++)if((*PY)(i)-d>=0||(*PX)(i)+b<=0)break;
				if(i==PX->nl){
				TMatrix X2(PX->nl,1),Y2(PX->nl,1);
				for(i=0;i<PX->nl;i++){X2(i)=logl((*PX)(i)+b);Y2(i)=logl(-(*PY)(i)+d);};
				stat=Stats(&X2,&Y2,0,PX->nl-1);
				if(stat.s1==0){imp=1;expr="Power model unadapted.";return;};
				c=stat.cov/stat.s1/stat.s1;
				}else{
				TMatrix Z(PX->nl-2,2);PTMatrix PIY=Integre(PX,PY);
				for(i=0;i<PX->nl-2;i++){(*PIY)(i)-=d*(*PX)(i+1);
					Z(i,0)=1;Z(i,1)=((*PX)(i+1)+b)*((*PY)(i+1)-d);
					};
				TMatrix Z2=~(!Z*Z);
				if(Z2==MatrixNull){imp=1;expr="Power model unadapted.";return;};
				TMatrix B=Z2*!Z*(*PIY);
				if(B(1)==0){imp=1;expr="Power model unadapted.";return;};
				c=1/B(1)-1;
				};};
			}else{PTMatrix PIY=Integre(PX,PY);
		if(!ci&&!di&&!bi){
			TMatrix Z(PX->nl-2,4);
			for(i=0;i<PX->nl-2;i++){
				Z(i,0)=1;Z(i,1)=(*PX)(i+1);Z(i,2)=(*PY)(i+1);
				Z(i,3)=(*PX)(i+1)*(*PY)(i+1);
				};
			TMatrix Z2=~(!Z*Z);
			if(Z2==MatrixNull){imp=1;expr="Power model unadapted.";return;};
			TMatrix B=Z2*!Z*(*PIY);
			if(B(3)==0){imp=1;expr="Power model unadapted.";return;};
			c=1/B(3)-1;b=B(2)/B(3);
			}else if(!ci&&!di&&bi){
			TMatrix Z(PX->nl-2,3);
			for(i=0;i<PX->nl-2;i++){
				Z(i,0)=1;Z(i,1)=(*PX)(i+1);
				Z(i,2)=((*PX)(i+1)+b)*(*PY)(i+1);
				};
			TMatrix Z2=~(!Z*Z);
			if(Z2==MatrixNull){imp=1;expr="Power model unadapted.";return;};
			TMatrix B=Z2*!Z*(*PIY);
			if(B(2)==0||B(2)==1){imp=1;expr="Power model unadapted.";return;};
			c=1/B(2)-1;
			}else if(!ci&&di&&!bi){
			TMatrix Z(PX->nl-2,3);
			for(i=0;i<PX->nl-2;i++){(*PIY)(i)-=d*(*PX)(i+1);
				Z(i,0)=1;Z(i,1)=(*PY)(i+1)-d;
				Z(i,2)=(*PX)(i+1)*((*PY)(i+1)-d);
				};
			TMatrix Z2=~(!Z*Z);
			if(Z2==MatrixNull){imp=1;expr="Power model unadapted.";return;};
			TMatrix B=Z2*!Z*(*PIY);
			if(B(2)==0){imp=1;expr="Power model unadapted.";return;};
			c=1/B(2)-1;b=B(1)/B(2);
			}else if(ci&&!bi&&!di){
			TMatrix Z(PX->nl-2,3);
			for(i=0;i<PX->nl-2;i++){(*PIY)(i)-=(*PX)(i+1)*(*PY)(i+1)/(c+1);
				Z(i,0)=1;Z(i,1)=(*PY)(i+1);
				Z(i,2)=(*PX)(i+1);
				};
			TMatrix Z2=~(!Z*Z);
			if(Z2==MatrixNull||c==0){imp=1;expr="Power model unadapted.";return;};
			TMatrix B=Z2*!Z*(*PIY);
			b=B(1)*(c+1);
			}else if(ci&&di){
			TMatrix Z(PX->nl-2,2);
			for(i=0;i<PX->nl-2;i++){(*PIY)(i)-=(*PX)(i+1)*(*PY)(i+1)/(c+1)+d*(*PX)(i+1)*c/(c+1);
				Z(i,0)=1;Z(i,1)=(*PY)(i+1);
				};
			TMatrix Z2=~(!Z*Z);
			if(Z2==MatrixNull){imp=1;expr="Power model unadapted.";return;};
			TMatrix B=Z2*!Z*(*PIY);
			b=B(1)*(c+1);
			}else if(ci&&bi){
			};
		};  //b, c et d sont alors connus
	if(!ai&&!di){
		TMatrix Y2(PX->nl,1);
		for(i=0;i<PX->nl;i++)if((*PX)(i)+b<0)
			{imp=1;expr="Power model unadapted.";return;}else
		  if((*PX)(i)+b==0)Y2(i)=0;else Y2(i)=powl((*PX)(i)+b,c);
		stat=Stats(&Y2,PY,0,PX->nl-1);
		if(stat.s1==0){imp=1;expr="Power model unadapted.";return;}
    a=stat.cov/stat.s1/stat.s1;d=stat.moy2-a*stat.moy1;
		}else if(ai&&!di){
		TMatrix Y2(PX->nl,1);
		for(i=0;i<PX->nl;i++)if((*PX)(i)+b<0)
			{imp=1;expr="Power model unadapted.";return;}else
		  if((*PX)(i)+b==0)Y2(i)=0;else Y2(i)=a*powl((*PX)(i)+b,c);
		stat=Stats(&Y2,PY,0,PX->nl-1);d=stat.moy2-a*stat.moy1;
		}else if(!ai&&di){
		TMatrix Y2(PX->nl,1),Y4=*PY-TMatrix(PX->nl,1,d);
		for(i=0;i<PX->nl;i++)if((*PX)(i)+b<0)
			{imp=1;expr="Power model unadapted.";return;}else
		  if((*PX)(i)+b==0)Y2(i)=0;else Y2(i)=powl((*PX)(i)+b,c);
		a=Coeff(&Y2,&Y4);
		};
	};      // a,b,c,d connus
TMatrix Y3(PX->nl,1);
for(i=0;i<PX->nl;i++)if((*PX)(i)+b<0)
	{imp=1;expr="Power model unadapted.";return;}else
  if((*PX)(i)+b==0)Y3(i)=d;else Y3(i)=a*powl((*PX)(i)+b,c)+d;
stat=Stats(PY,&Y3,0,PX->nl-1);
delete[] expr;expr=new char[170];
sprintf(expr,"Y=a (X + b)^c + d\na=%g\nb=%g\nc=%g\nd=%g",a,b,c,d);
}

TVal TModPuis::f(double x)
{TVal Val;if(imp||x+b<0)Val.IsDefined=0;else{
	if(x+b>0)Val.v=a*powl(x+b,c)+d;else Val.v=d;
	Val.IsDefined=1;
	};
return Val;
}

void TModSin::Etablit(PTMatrix PX,PTMatrix PY)
{int i;imp=0;
if(!bi){
	PTMatrix PIY=Integre(PX,PY);
  if(di){
		TMatrix Y2(PX->nl-2,1),Z(PX->nl-2,3);
		for(i=0;i<PX->nl-2;i++){
			Y2(i)=((*PY)(i+1)-d*(*PX)(i+1))*((*PY)(i+1)-d*(*PX)(i+1));
			Z(i,0)=1;
			Z(i,1)=(*PIY)(i)-d*(*PX)(i+1);
			Z(i,2)=((*PIY)(i)-d*(*PX)(i+1))*((*PIY)(i)-d*(*PX)(i+1));
			};
		TMatrix Z2=~(!Z*Z);
		if(Z2==MatrixNull){imp=1;expr="Sinusoidal model unadapted.";return;};
		TMatrix B=Z2*!Z*Y2;
		if(B(2)>0){imp=1;expr="Sinusoidal model unadapted.";return;};
		b=sqrtl(-B(2));}else{
		TMatrix Y2(PX->nl-2,1),Z(PX->nl-2,7);
		for(i=0;i<PX->nl-2;i++){
			Y2(i)=(*PY)(i+1)*(*PY)(i+1);
			Z(i,0)=1;Z(i,1)=(*PX)(i+1);Z(i,2)=(*PX)(i+1)*(*PX)(i+1);
			Z(i,3)=(*PIY)(i);Z(i,4)=(*PX)(i+1)*(*PIY)(i);
			Z(i,5)=(*PIY)(i)*(*PIY)(i);Z(i,6)=(*PY)(i+1);
			};
		TMatrix Z2=~(!Z*Z);
		if(Z2==MatrixNull){imp=1;expr="Sinusoidal model unadapted.";return;};
		TMatrix B=Z2*!Z*Y2;
		if(B(5)>0){imp=1;expr="Sinusoidal model unadapted.";return;};
		b=sqrtl(-B(5));
		}; //b connu
  };
if(di&&!ci){
	TMatrix Y2=(*PY)-TMatrix(PX->nl,1,d);
	TMatrix Z(PX->nl,2);
	for(i=0;i<PX->nl;i++){
		Z(i,0)=sinl(b*(*PX)(i));
		Z(i,1)=cosl(b*(*PX)(i));
		};
  TMatrix Z2=~(!Z*Z);
	if(Z2==MatrixNull){imp=1;expr="Sinusoidal model unadapted.";return;};
	TMatrix B=Z2*!Z*Y2;
	if(B(0)==0&&B(1)==0){imp=1;expr="Sinusoidal model unadapted.";return;};
	if(ai&&a<0){B(0)=-B(0);B(1)=-B(1);};c=atan2l(B(1),B(0));
	if(!ai)a=sqrtl(B(0)*B(0)+B(1)*B(1));
	};
if(di&&ci&&!ai){
	TMatrix Y2=(*PY)-TMatrix(PX->nl,1,d);
  TMatrix Y3(PX->nl,1);
	for(i=0;i<PX->nl;i++)Y3(i)=sinl(b*(*PX)(i)+c);
	a=Coeff(&Y3,&Y2);
	};
if(!di&&!ci){
	TMatrix Z(PX->nl,3);
	for(i=0;i<PX->nl;i++){
		Z(i,0)=sinl(b*(*PX)(i));
		Z(i,1)=cosl(b*(*PX)(i));Z(i,2)=1;
		};
  TMatrix Z2=~(!Z*Z);
	if(Z2==MatrixNull){imp=1;expr="Sinusoidal model unadapted.";return;};
	TMatrix B=Z2*!Z*(*PY);
	if(B(0)==0&&B(1)==0){imp=1;expr="Sinusoidal model unadapted.";return;};
	if(ai&&a<0){B(0)=-B(0);B(1)=-B(1);};c=atan2l(B(1),B(0));
	if(!ai)a=sqrtl(B(0)*B(0)+B(1)*B(1));
  d=B(2);
  };
if(!di&&ci&&!ai){
	TMatrix Y2(PX->nl,1);
	for(i=0;i<PX->nl;i++)Y2(i)=sinl(b*(*PX)(i)+c);
	stat=Stats(&Y2,PY,0,PX->nl-1);
	if(stat.s1==0){imp=1;expr="Sinusoidal model unadapted.";return;};
	a=stat.cov/stat.s1/stat.s1;d=stat.moy2-a*stat.moy1;
  };
if(!di&&ci&&ai){
	for(d=0,i=0;i<PX->nl;i++)d+=(*PY)(i)-a*sinl(b*(*PX)(i)+c);
	d/=PX->nl;
  };
TMatrix Y3(PX->nl,1);
for(i=0;i<PX->nl;i++)Y3(i)=a*sinl(b*(*PX)(i)+c)+d;
stat=Stats(PY,&Y3,0,PX->nl-1);
delete[] expr;expr=new char[170];
sprintf(expr,"Y=a sin(b X + c) + d\na=%g\nb=%g\nc=%g\nd=%g",a,b,c,d);
}

TVal TModSin::f(double x)
{TVal Val;if(imp)Val.IsDefined=0;else{
	Val.v=a*sinl(x*b+c)+d;
	Val.IsDefined=1;
	};
return Val;
}
