#ifndef __STDLIB_H
#include<stdlib.h>
#endif

#ifndef __STDIO_H
#include<stdio.h>
#endif

#ifndef __MATH_H
#include<math.h>
#endif

#ifndef __STRING_H
#include<string.h>
#endif

#ifndef __MATRICES_H
#include"matrices.h"
#endif
                                       
#define imax(a,b) (a>b?a:b)

using namespace std;


TMatrix::TMatrix(const TMatrix & m)
{nl=m.nl;nc=m.nc;
PMatrix=new long double [nl*nc?nl*nc:1];
if(PMatrix==NULL){nl=0;nc=0;return;};
long double *f1=PMatrix,*f2=m.PMatrix;
for(;f1<PMatrix+nl*nc;)*(f1++)=*(f2++);
}

TMatrix::TMatrix(int nlp,int ncp,long double *PMatrixp)
{nl=nlp;nc=ncp;if(nl<0)nl=0;if(nc<0)nc=0;
PMatrix=new long double [nl*nc?nl*nc:1];
long double *f1=PMatrix,*f2=PMatrixp;
for(;f1<PMatrix+nl*nc;)*(f1++)=*(f2++);
}

TMatrix::TMatrix(int nlp,int ncp,long double Initp)  //If Initp!=0, the function must be called as TMatrix(nlp,ncp,(long double)Initp) in order to avoid a confusion with TMatrix(int,int,long double *)
{nl=nlp;nc=ncp;if(nl<0)nl=0;if(nc<0)nc=0;
 PMatrix=new long double [nl*nc?nl*nc:1];
 long double *f1=PMatrix;
 for(;f1<PMatrix+nl*nc;)*(f1++)=Initp;
}

int TMatrix::Resize(int nlp,int ncp,long double Initp=0)  //The return value is nonzero if and only if the function is successful.
{if(nlp<0||ncp<0)return 0;
 if(nlp==nl&&ncp==nc)return 2;
long double *PMatrix2=new long double [nlp*ncp?nlp*ncp:1];
if(PMatrix2==NULL)return 0;
int i,j;
for(i=0;i<nlp;i++)for(j=0;j<ncp;j++)*(PMatrix2+i*ncp+j)=Initp;
for(i=0;i<nl&&i<nlp;i++)for(j=0;j<nc&&j<ncp;j++)
	*(PMatrix2+i*ncp+j)=a(i,j);
delete[]PMatrix;PMatrix=PMatrix2;
nl=nlp;nc=ncp;
return 1;
}

int TMatrix::DelRow(int i,int n=1)  //The return value is nonzero if and only if the function is successful
{if(i<0||i>=nl||nl==0||i+n>nl||n<=0)return 0;
int i2,j2;
for(i2=i;i2<nl-n;i2++)for(j2=0;j2<nc;j2++)Seta(i2,j2,a(i2+n,j2));
return Resize(nl-n,nc);
}

int TMatrix::DelCol(int j,int n=1)  //The return value is nonzero if and only if the function is successful
{if(j<0||j>nc||nc==0||n<=0||j+n>nc)return 0;
int i2,j2;
for(j2=j;j2<nc-n;j2++)for(i2=0;i2<nl;i2++)Seta(i2,j2,a(i2,j2+n));
return Resize(nl,nc-n);
}

int TMatrix::InsRow(int i,int n=1)  //The return value is nonzero if and only if the function is successful
{if(i<0||i>=nl||n<=0)return 0;
if(!Resize(nl+n,nc))return 0;
int i2,j2;
for(i2=nl-1;i2>=i+n;i2--)for(j2=0;j2<nc;j2++)Seta(i2,j2,a(i2-n,j2));
for(i2=i;i2<i+n;i2++)for(j2=0;j2<nc;j2++)Seta(i2,j2,0);
return 1;
}

int TMatrix::InsCol(int j,int n=1)  //The return value is nonzero if and only if the function is successful
{if(j<0||j>=nc||n<=0)return 0;
if(!Resize(nl,nc+n))return 0;
int i2,j2;
for(j2=nc-1;j2>=j+n;j2--)for(i2=0;i2<nl;i2++)Seta(i2,j2,a(i2,j2-n));
for(j2=j;j2<j+n;j2++)for(i2=0;i2<nl;i2++)Seta(i2,j2,0);
return 1;
}

TMatrix& TMatrix::operator=(const TMatrix& m)
{
 if(this==&m)return *this;
 nl=m.nl;nc=m.nc;
 delete[]PMatrix;
 PMatrix=new long double [nl*nc?nl*nc:1];
 //if(PMatrix==NULL)return MatrixNull;
 long double *f1=PMatrix,*f2=m.PMatrix;
 for(;f1<PMatrix+nl*nc;)*(f1++)=*(f2++);
 return *this;
}

int operator==(const TMatrix& m1,const TMatrix& m2)
{if(m1.nl!=m2.nl||m1.nc!=m2.nc)return 0;
long double *f1=m1.PMatrix,*f2=m2.PMatrix;
for(;f1<m1.PMatrix+m1.nl*m1.nc;)if(*(f1++)!=*(f2++))return 0;
return 1;
}

int operator!=(const TMatrix& m1,const TMatrix& m2)
{if(m1.nl!=m2.nl||m1.nc!=m2.nc)return 1;
long double *f1=m1.PMatrix,*f2=m2.PMatrix;
for(;f1<m1.PMatrix+m1.nl*m1.nc;)if(*(f1++)!=*(f2++))return 1;
return 0;
}

inline TMatrix TMatrix::operator+()const
{return *this;}

TMatrix TMatrix::operator-()const
{TMatrix m(nl,nc);
long double *f1=m.PMatrix,*f2=PMatrix;
for(;f1<m.PMatrix+m.nl*m.nc;)*(f1++)=-*(f2++);
return m;
}

TMatrix operator+(const TMatrix& m1,const TMatrix& m2)
{
 int nc=imax(m1.nc,m2.nc),nl=imax(m1.nl,m2.nl);
 int i=0,j=0;
 TMatrix m3(nl,nc);
 if(m1.nc==m2.nc&&m1.nl==m2.nl){
	 for(;i<nl;i++)for(j=0;j<nc;j++)m3.Seta(i,j,m1.a(i,j)+m2.a(i,j));
 }else{
	 TMatrix m1c(m1),m2c(m2);m1c.Resize(nl,nc);m2c.Resize(nl,nc);
	 for(;i<nl;i++)for(j=0;j<nc;j++)m3.Seta(i,j,m1c.a(i,j)+m2c.a(i,j));
 }
return m3;
}

TMatrix operator+(const long double& k,const TMatrix& m)	//m+k*Id in case of a square matrix
{TMatrix m2=m;
if(m.nl!=m.nc)return m2;
long double *f=m2.PMatrix;
for(;f<m2.PMatrix+m2.nl*m2.nl;f+=m2.nl+1)*f+=k;
return m2;
}

TMatrix operator+(const TMatrix& m,const long double& k)  //m+k*Id in case of a square matrix
{TMatrix m2=m;
if(m.nl!=m.nc)return m2;
long double *f=m2.PMatrix;
for(;f<m2.PMatrix+m2.nl*m2.nl;f+=m2.nl+1)*f+=k;
return m2;
}

TMatrix operator-(const TMatrix& m1,const TMatrix& m2)
{
 int nc=imax(m1.nc,m2.nc),nl=imax(m1.nl,m2.nl);
 int i=0,j=0;
 TMatrix m3(nl,nc);
 if(m1.nc==m2.nc&&m1.nl==m2.nl){
	 for(;i<nl;i++)for(j=0;j<nc;j++)m3.Seta(i,j,m1.a(i,j)-m2.a(i,j));
 }else{
	 TMatrix m1c(m1),m2c(m2);m1c.Resize(nl,nc);m2c.Resize(nl,nc);
	 for(;i<nl;i++)for(j=0;j<nc;j++)m3.Seta(i,j,m1c.a(i,j)-m2c.a(i,j));
 }
 return m3;
}

TMatrix operator*(const long double& k,const TMatrix& m)
{TMatrix m2(m.nl,m.nc);
long double *f1=m2.PMatrix,*f2=m.PMatrix;
for(;f1<m2.PMatrix+m2.nl*m2.nc;)*(f1++)=k*(*(f2++));
return m2;
}

TMatrix operator*(const TMatrix& m1,const TMatrix& m2)
{
 const TMatrix *m1c=&m1,*m2c=&m2;
 if(m1.nc!=m2.nl)printf("Pb de taille!\n");
/* if(m1.nc<m2.nl){
	 m1c=new TMatrix(m1);
	 m1c->Resize(m1.nl,m2.nl);printf("Pb de taille!\n");
 }else if(m1.nc>m2.nl){
	 m2c=new TMatrix(m2);
	 m2c->Resize(m1.nc,m2.nc);printf("Pb de taille !\n");
 }*/
 TMatrix m3(m1c->nl,m2c->nc);
 int i,j,k;
/* for(i=0;i<m1c->nl;i++)for(j=0;j<m2c->nc;j++)
	 for(k=0;k<m1c->nc;k++)*(m3.Pa(i,j))+=m1c->a(i,k)*m2c->a(k,j);*/
 long double d;
 for(i=0;i<m1c->nl;i++)for(k=0;k<m1c->nc;k++){
	 d=m1c->a(i,k);
	 if(d!=0.0)for(j=0;j<m2c->nc;j++)*(m3.Pa(i,j))+=d*m2c->a(k,j);
 }
/* if(m1c!=&m1)delete m1c;
 if(m2c!=&m2)delete m2c;*/
 return m3;
}

TMatrix& TMatrix::operator+=(const TMatrix& m2)
{TMatrix m3=m2;m3.Resize(nl,nc);  //TMatrix m2 gets the size of the TMatrix it is added to
int i,j;
for(i=0;i<nl;i++)for(j=0;j<nc;j++)*Pa(i,j)+=m3.a(i,j);
return *this;
}

TMatrix& TMatrix::operator+=(const long double& k)	//m+k*Id in case of a square matrix
{if(nl!=nc)return *this;
long double *f=PMatrix;
for(;f<PMatrix+nl*nl;f+=nl+1)*f+=k;
return *this;
}

TMatrix& TMatrix::operator-=(const TMatrix& m2)
{TMatrix m3=m2;m3.Resize(nl,nc);  //TMatrix m2 gets the size of the TMatrix it is added to
int i,j;
for(i=0;i<nl;i++)for(j=0;j<nc;j++)*Pa(i,j)-=m3.a(i,j);
return *this;
}

TMatrix& TMatrix::operator*=(const long double& k)
{long double *f1=PMatrix;
for(;f1<PMatrix+nl*nc;)*(f1++)*=k;
return *this;
}

TMatrix& TMatrix::operator*=(const TMatrix& m2)
{return (*this=(*this)*m2);
}

TMatrix TMatrix::operator!()const
{TMatrix m2(nc,nl);
int i,j;for(i=0;i<nc;i++)for(j=0;j<nl;j++)*m2.Pa(i,j)=a(j,i);
return m2;
}

TMatrix TMatrix::operator~()const
{
 if(nl!=nc)return MatrixNull;
 if(nl==1&&nc==1)if(a(0,0)!=0)return TMatrix(1,1,(long double)1./a(0,0));else return MatrixNull;
 int n=nl;if(nc!=n)return MatrixNull;
 TMatrix LU=*this;int*Pvect;if(!PLUdec(LU,Pvect)){delete[]Pvect;return MatrixNull;}
 TMatrix y(n);TMatrix I(n,n);
 int i,j,k;
 TMatrix bvect(n);
 for(k=0;k<n;k++){
	 if(k)bvect(k-1)=0;
	 bvect(k)=1;
	 for(i=0;i<n;i++){
		 y(i)=bvect(Pvect[i]);
		 for(j=0;j<i;j++)y(i)-=LU(i,j)*y(j);
	 }
	 for(i=n-1;i>=0;i--){
		 I(i,k)=y(i);
		 for(j=i+1;j<n;j++)I(i,k)-=LU(i,j)*I(j,k);
		 I(i,k)/=LU(i,i);
	 }
 }
 delete[]Pvect;
 return I;
}

long double trace(const TMatrix&A)
{
 long double s=0;
 for(int i=0;i<A.nl&&i<A.nc;i++)s+=A(i,i);
 return s;
}

long double det(const TMatrix&A)
{
 if(A.nl!=A.nc)return 0;
 if(A.nl==1)return A(0,0);
 if(A.nl==2)return A(0,0)*A(1,1)-A(1,0)*A(0,1);
 TMatrix LU=A;int*PVect;if(!PLUdec(LU,PVect)){delete[]PVect;return 0;}
 long double d=1;
 int i,j,n=A.nl;
 for(i=0;i<n;i++)d*=LU(i,i);
 int s=1;
 for(i=0;i<n;i++)for(j=i+1;j<n;j++)if(PVect[i]>PVect[j])s=-s;
 delete[]PVect;
 return d*(long double)s;
}

int rank(const TMatrix& M)
{int nl=M.nl,nc=M.nc,r=0;
if(nl==1&&nc==1)return (M.a(0,0)!=0.?1:0);
TMatrix m2=M;
int i,j,k,k2;long double c,d;
for(j=0,i=0;j<nc&&i<nl;j++,i++){
	if(m2.a(i,j)==0.0){
	k=i+1;while(k<nl&&m2.a(k,j)==0.)k++;if(k==nl){i--;continue;};
	for(k2=j;k2<nc;k2++)*m2.Pa(i,k2)+=m2.a(k,k2);
	}
	r++;
	d=1./m2.a(i,j);for(k=i+1;k<nl;k++)
	if((c=m2.a(k,j))!=0){c*=d;for(k2=j;k2<nc;k2++)*m2.Pa(k,k2)-=m2.a(i,k2)*c;
		}
	}
return r;
}

TMatrix MatrixId(int n)
{TMatrix m(n,n);
long double *f=m.PMatrix;
for(;f<m.PMatrix+n*n;f+=n+1)*f=1;
return m;
}

long double max(const TMatrix& m)
{if(!(m.nl&&m.nl))return 0;
long double *f=m.PMatrix,a=*f;
for(;f<m.PMatrix+m.nl*m.nc;f++)if(a<*f)a=*f;
return a;
}

long double min(const TMatrix& m)
{if(!(m.nl&&m.nl))return 0;
long double *f=m.PMatrix,a=*f;
for(;f<m.PMatrix+m.nl*m.nc;f++)if(a>*f)a=*f;
return a;
}

TMatrix exp(const TMatrix& m)
{if(m.nl!=m.nc)return MatrixNull;
long double e=max(m);
int k,q=0,j=imax(0,1+(int)floor(log(e)*1.44269504088896340736));
TMatrix m2=(1./pow(2.,j))*m;
long double c=8;
while(c>1e-20*e){q++;c*=0.0625/(2*q-1)/(2*q+1);}
TMatrix d=MatrixId(m.nc),n=MatrixId(m.nc),x=MatrixId(m.nc);
c=1.;
for(k=1;k<=q;k++){
	c*=1./k*(q-k+1)/(2*q-k+1);
	x=m2*x;
	n+=c*x;
	d+=(k%2?-1.:1.)*c*x;
	}
TMatrix f=(~d)*n;
for(k=0;k<j;k++)f=f*f;
return f;
}


bool LUdec(TMatrix& A)
{
 int n=A.nl;if(A.nc!=n)return 0;
 int i,j,k;long double e;
 for(k=0;k<n;k++){
	 e=A(k,k);if(e==0.0)return 0;
	 for(i=k+1;i<n;i++)A(i,k)/=e;
	 for(i=k+1;i<n;i++)for(j=k+1;j<n;j++)A(i,j)-=A(i,k)*A(k,j);
 }
 return 1;
}

bool PLUdec(TMatrix& A,int*& Pvect)
{
 int n=A.nl;
 Pvect=new int[n];
 if(A.nc!=n)return 0;
 int i,j,k,k2,k3;long double p;long double e;
 for(i=0;i<n;i++)Pvect[i]=i;
 for(k=0;k<n;k++){
	 for(p=0.,i=k;i<n;i++)if(fabs(A(i,k))>p){p=fabs(A(i,k));k2=i;}
	 if(p==0.)return 0;
	 k3=Pvect[k];Pvect[k]=Pvect[k2];Pvect[k2]=k3;
	 for(i=0;i<n;i++){
		 e=A(k,i);A(k,i)=A(k2,i);A(k2,i)=e;
	 }
	 e=A(k,k);
	 for(i=k+1;i<n;i++){
		 A(i,k)/=e;
		 for(j=k+1;j<n;j++)A(i,j)-=A(i,k)*A(k,j);
	 }
 }
 return 1;
}

TMatrix Solve(const TMatrix& A,const TMatrix& bvect)
{
 int n=bvect.nl;if(A.nl!=n)return MatrixNull;
 if(A.nc<n)return MatrixNull;
 if(A.nc==n){
	 TMatrix LU=A;int* Pvect;if(!PLUdec(LU,Pvect)){delete[]Pvect;return MatrixNull;}
	 TMatrix y(n),x(n);
	 int i,j;
	 for(i=0;i<n;i++){
		 y(i)=bvect(Pvect[i]);
		 for(j=0;j<i;j++)y(i)-=LU(i,j)*y(j);
	 }
	 for(i=n-1;i>=0;i--){
		 x(i)=y(i);
		 for(j=i+1;j<n;j++)x(i)-=LU(i,j)*x(j);
		 x(i)/=LU(i,i);
	 }
	 delete[]Pvect;
	 return x;
 }else{
	 TMatrix tA=!A,B=~(A*tA);if(B==MatrixNull)return MatrixNull;
	 return tA*B*bvect;
 }
}

void GramSchmidtify(TMatrix& A)
{
 int i,j,k;long double s;
 for(j=0;j<A.nc;j++){
	 for(k=0;k<j;k++){
		 for(s=0,i=0;i<A.nl;i++)s+=A(i,j)*A(i,k);
		 if(s==0.)continue;
		 for(i=0;i<A.nl;i++)A(i,j)-=s*A(i,k);
	 }
	 s=0;
	 for(i=0;i<A.nl;i++)s+=A(i,j)*A(i,j);
	 if(s==0.)continue;
	 s=sqrt(s);
	 for(i=0;i<A.nl;i++)A(i,j)/=s;
 }
}

bool LUPdec(TMatrix& A,int*& Pvect)//This one will not complain with a non-square matrix
{
 int n=A.nc;
 Pvect=new int[n];
if(A.nl>n)return 0;
 int i,j,k,k2,k3;long double p,e;
 for(i=0;i<n;i++)Pvect[i]=i;
 for(k=0;k<A.nl;k++){
	 for(p=0.,i=k;i<n;i++)if(fabs(A(k,i))>p){p=fabs(A(k,i));k2=i;}
	 if(p==0.)return 0;
	 k3=Pvect[k];Pvect[k]=Pvect[k2];Pvect[k2]=k3;
	 for(i=0;i<A.nl;i++){
		 e=A(i,k);A(i,k)=A(i,k2);A(i,k2)=e;
	 }
	 e=A(k,k);
	 for(i=k+1;i<n;i++){
		 A(k,i)/=e;
		 for(j=k+1;j<A.nl;j++)A(j,i)-=A(k,i)*A(j,k);
	 }
 }
 return 1;
}

/*
TMatrix AutreSolve(const TMatrix& A,const TMatrix& bvect)
{
 int n=bvect.nl;if(A.nl!=n||A.nc!=n)return MatrixNull;
 TMatrix LU=A;int* Pvect;if(!LUPdec(LU,Pvect)){delete[]Pvect;return MatrixNull;}
 TMatrix y(n),x(n);
 int i,j;
 for(i=0;i<n;i++){
	 y(i)=bvect(i);
	 for(j=0;j<i;j++)y(i)-=LU(i,j)*y(j);
	 y(i)/=LU(i,i);
 }
 for(i=n-1;i>=0;i--){
	 x(i)=y(i);
	 for(j=i+1;j<n;j++)x(i)-=LU(i,j)*x(j);
 }
 TMatrix z(n);
 for(i=0;i<n;i++)z(Pvect[i])=x(i);
 delete[]Pvect;
 return z;
}
*/

TMatrix kerbase(const TMatrix& A)//Works only if nl<nc and A surjective; returns a (A.nc,A.nc-A.nl)-matrix
{
 if(A.nl>=A.nc)return MatrixNull;
 TMatrix LU=A;int*Pvect;if(!LUPdec(LU,Pvect)){delete[]Pvect;return MatrixNull;}
 int i,j,k;
 TMatrix LU2(A.nc,A.nc);
 for(i=0;i<A.nl;i++)for(j=0;j<A.nc;j++)LU2(i,j)=LU(i,j);
 for(i=A.nl;i<A.nc;i++)LU2(i,i)=1;
 TMatrix bvect(A.nc),y(A.nc),I(A.nc,A.nc-A.nl);
 for(k=0;k<A.nc-A.nl;k++){
	 if(k)bvect(A.nl+k-1)=0;
	 bvect(A.nl+k)=1;
	 for(i=0;i<A.nl;i++){
		 y(i)=bvect(i);
		 for(j=0;j<i;j++)y(i)-=LU2(i,j)*y(j);
		 y(i)/=LU2(i,i);
	 }
//	 for(i=A.nl;i<A.nc;i++)y(i)=bvect(i);
	 for(i=A.nc-1;i>=A.nl;i--)I(i,k)=bvect(i);//y(i);
	 for(i=A.nl-1;i>=0;i--){
		 I(i,k)=y(i);
		 for(j=i+1;j<A.nc;j++)I(i,k)-=LU(i,j)*I(j,k);
	 }
 }
 TMatrix KB(I.nl,I.nc);
 for(i=0;i<KB.nl;i++)for(j=0;j<KB.nc;j++)
	 KB(Pvect[i],j)=I(i,j);
 delete[]Pvect;
 return KB;
}
